/// ==============================================================================================
/// Namespace   : ScipBe.Controls.WinForms
/// Class       : FontItemDrawEngine, SimpleFontItemDrawEngine, ModernFontItemDrawEngine
/// Version     : 1.1
/// Author      : Stefan Cruysberghs
/// Website     : www.scip.be
/// Date        : September 2006 - May 2007
/// Description : Parent FontItemDrawEngine for FontCombobox and derived components
/// Status      : Open source - Mozilla Public License 1.1
/// ==============================================================================================

using System.ComponentModel;
using System.Drawing;
using System.Threading;
using System.Windows.Forms;
using System.Windows.Forms.VisualStyles;

namespace ScipBe.Controls.WinForms
{
  #region IFontItemDrawEngine interface
  /// <summary>
  /// Interface for FontItemDrawEngine
  /// </summary>
  interface IFontItemDrawEngine
  {
    int CategoryHeight { get; }
    void DrawCategory(System.Windows.Forms.Control control, FontItem fontItem, System.Windows.Forms.DrawItemEventArgs e);
    void DrawItem(System.Windows.Forms.Control control, FontItem fontItem, System.Windows.Forms.DrawItemEventArgs e);
    int ItemHeight { get; }
  }
  #endregion

  #region FontItemDrawEngine abstract class
  /// <summary>
  /// Abstract class for FontItemDrawEngine which contains draw events and item heights
  /// </summary>
  public abstract class FontItemDrawEngine : Component, IFontItemDrawEngine
  {
    public abstract void DrawItem(Control control, FontItem fontItem, DrawItemEventArgs e);
    public abstract void DrawCategory(Control control, FontItem fontItem, DrawItemEventArgs e);
    protected int itemHeight;
    public int ItemHeight
    {
      get
      {
        return itemHeight;
      }
    }
    protected int categoryHeight;
    public int CategoryHeight
    {
      get
      {
      	return categoryHeight;
      }
    }
  }
  #endregion

  #region SimpleFontItemDrawEngine class
  /// <summary>
  /// Simple FontItemDrawEngine which only shows the name of the font 
  /// </summary>
  [ToolboxBitmap(typeof(ResourceFinder), "ScipBe.Controls.WinForms.FontComboManager.Resources.SimpleFontItemDrawEngine.ico")]
  [Category("ScipBe")]
  public class SimpleFontItemDrawEngine : FontItemDrawEngine
  {
    private string defaultFontName;

    public SimpleFontItemDrawEngine()
    {
      categoryHeight = 22;
      itemHeight = 24;
      defaultFontName = "Calibri";
    }

    override public void DrawItem(Control control, FontItem fontItem, DrawItemEventArgs e)
    {
      Brush textColor = Brushes.Black;
      Brush styleColor = Brushes.Silver;

      e.DrawBackground();

      StringFormat format1 = new StringFormat(StringFormatFlags.NoWrap);
      format1.LineAlignment = StringAlignment.Center;
      format1.Alignment = StringAlignment.Near;

      Font fontName = new Font(defaultFontName, 10);
      e.Graphics.DrawString(fontItem.Family.Name, fontName, Brushes.Black, e.Bounds, format1);
    }

    override public void DrawCategory(Control control, FontItem fontItem, DrawItemEventArgs e)
    {
      Font font = new Font(defaultFontName, 10); 

      e.Graphics.FillRectangle(new SolidBrush(
        SystemColors.ButtonFace),
        e.Bounds.X, e.Bounds.Y + 5,
        e.Bounds.Width, e.Bounds.Height - 10);
      e.Graphics.FillRectangle(new SolidBrush(
        SystemColors.ButtonShadow),
        e.Bounds.X, e.Bounds.Y + e.Bounds.Height - 5,
        e.Bounds.Width, 1);

      StringFormat format1 = new StringFormat(StringFormatFlags.NoWrap);
      format1.LineAlignment = StringAlignment.Center;
      format1.Alignment = StringAlignment.Near;

      e.Graphics.DrawString(fontItem.Description, font, Brushes.Gray, e.Bounds, format1);
    }
  }
  #endregion

  #region ModernFontItemDrawEngine class
  /// <summary>
  /// Modern FontItemDrawEngine which shows fontname, font preview, font properties 
  /// and which uses XP/Vista theme support
  /// </summary>
  [ToolboxBitmap(typeof(ResourceFinder), "ScipBe.Controls.WinForms.FontComboManager.Resources.ModernFontItemDrawEngine.ico")]
  [Category("ScipBe")]
  public class ModernFontItemDrawEngine : FontItemDrawEngine
  {
    private string defaultFontName;
    
    public ModernFontItemDrawEngine()
    {
      categoryHeight = 24;
      itemHeight = 46;
      if (Thread.CurrentThread.CurrentUICulture.ThreeLetterISOLanguageName == "nld")
        pangramText = "Pa's wijze lynx bezag vroom het fikse aquaduct";
      else
        pangramText = "The quick brown fox jumps over the lazy dog";

      defaultFontName = "Calibri";
    }
      
    private string pangramText;
    [Localizable(true)]
    public string PangramText
    {
        get { return pangramText; }
        set { pangramText = value; }
    }

    override public void DrawItem(Control control, FontItem fontItem, DrawItemEventArgs e)
    {
      Brush textColor = Brushes.Black;
      Brush styleColor = Brushes.Silver;

      e.Graphics.FillRectangle(Brushes.White, new Rectangle(0, e.Bounds.Y, e.Bounds.Width, e.Bounds.Height));

      if (e.State == DrawItemState.Selected)
      {
        textColor = Brushes.Black; 
        styleColor = Brushes.Black;
      }

      if ((e.State & DrawItemState.Selected) == DrawItemState.Selected)
      {
        // Check if XP theme is available
        if (Application.RenderWithVisualStyles)
        {
          VisualStyleElement element = VisualStyleElement.TaskBand.FlashButton.Normal;
          VisualStyleRenderer renderer = new VisualStyleRenderer(element);
          e.Bounds.Inflate(new Size(2, 2));
          renderer.DrawBackground(e.Graphics, e.Bounds);
          e.Bounds.Inflate(new Size(-2, -2));
        }
        else
        {
          e.DrawBackground();
        }
      }
      else
      {
        e.DrawBackground();
      }

      Font fontLarge;
      if (fontItem.Family.IsStyleAvailable(FontStyle.Regular))
      {
        fontLarge = new Font(fontItem.Family, 20);
      }
      else if (fontItem.Family.IsStyleAvailable(FontStyle.Italic))
      {
        fontLarge = new Font(fontItem.Family, 20, FontStyle.Italic);
      }
      else if (fontItem.Family.IsStyleAvailable(FontStyle.Bold))
      {
        fontLarge = new Font(fontItem.Family, 20, FontStyle.Bold);
      }
      else
      {
        fontLarge = null;
      }

      StringFormat format1 = new StringFormat(StringFormatFlags.NoWrap);
      format1.LineAlignment = StringAlignment.Center;
      format1.Alignment = StringAlignment.Center;

      e.Graphics.FillRectangle(Brushes.White, new Rectangle(3, e.Bounds.Y + 5, 75, e.Bounds.Height - 10));
      e.Graphics.DrawRectangle(new Pen(textColor, 1), new Rectangle(3, e.Bounds.Y + 5, 75, e.Bounds.Height - 10));
      if (fontLarge != null)
      {
        //int fontHeight = fontLarge.GetHeight();
        e.Graphics.DrawString("Aa", fontLarge, textColor, new Rectangle(3, e.Bounds.Y, 75, e.Bounds.Height), format1);
      }

      if ((e.State & DrawItemState.Selected) == DrawItemState.Selected)
      {
        Font fontSmall;
        if (fontItem.Family.IsStyleAvailable(FontStyle.Regular))
        {
          fontSmall = new Font(fontItem.Family, 12);
        }
        else if (fontItem.Family.IsStyleAvailable(FontStyle.Italic))
        {
          fontSmall = new Font(fontItem.Family, 12, FontStyle.Italic);
        }
        else if (fontItem.Family.IsStyleAvailable(FontStyle.Bold))
        {
          fontSmall = new Font(fontItem.Family, 12, FontStyle.Bold);
        }
        else
        {
          fontSmall = null;
        }
        if (fontSmall != null)
        {
          Font fontName = new Font(defaultFontName, 9);
          e.Graphics.DrawString(fontItem.Family.Name, fontName, textColor, e.Bounds.X + 85, e.Bounds.Y + 3);

          StringFormat format2 = new StringFormat(StringFormatFlags.NoWrap);
          format2.LineAlignment = StringAlignment.Far;
          format2.Alignment = StringAlignment.Near;
          format2.Trimming = StringTrimming.EllipsisCharacter;

          e.Graphics.DrawString(pangramText, fontSmall, textColor, new Rectangle(e.Bounds.X + 85, e.Bounds.Y + 16, e.Bounds.Width - 38 - 100, e.Bounds.Height - 16), format2);
        }
      }
      else
      {
        Font fontName = new Font(defaultFontName, 11);
        e.Graphics.DrawString(fontItem.Family.Name, fontName, textColor, e.Bounds.X + 85, e.Bounds.Y + 10);
      }

      e.Graphics.FillRectangle(Brushes.White, new Rectangle(e.Bounds.Width - 35, e.Bounds.Y + 30, 30, 11));
      Font fontRegular = new Font(defaultFontName, 10);
      if (fontItem.Family.IsStyleAvailable(FontStyle.Regular))
        e.Graphics.DrawString("R", fontRegular, styleColor, e.Bounds.Width - 35, e.Bounds.Y + 27);
      e.Graphics.DrawRectangle(new Pen(styleColor, 1), new Rectangle(e.Bounds.Width - 35, e.Bounds.Y + 30, 10, 11));
      if (fontItem.Family.IsStyleAvailable(FontStyle.Bold))
        e.Graphics.DrawString("B", fontRegular, styleColor, e.Bounds.Width - 25, e.Bounds.Y + 27);
      e.Graphics.DrawRectangle(new Pen(styleColor, 1), new Rectangle(e.Bounds.Width - 25, e.Bounds.Y + 30, 10, 11));
      if (fontItem.Family.IsStyleAvailable(FontStyle.Italic))
        e.Graphics.DrawString("I", fontRegular, styleColor, e.Bounds.Width - 15, e.Bounds.Y + 27);
      e.Graphics.DrawRectangle(new Pen(styleColor, 1), new Rectangle(e.Bounds.Width - 15, e.Bounds.Y + 30, 10, 11));

      if ((fontItem.Description != "") && (fontItem.Description != null))
      {
        Font fontDescription = new Font(defaultFontName, 10);
        Graphics graph = control.CreateGraphics();
        int xPos = (int)graph.MeasureString(fontItem.Description, fontDescription).Width;

        e.Graphics.DrawString("(" + fontItem.Description + ")", fontDescription, textColor,
          e.Bounds.X + e.Bounds.Width - xPos - 10, e.Bounds.Y + 6);
      }
    }
    
    override public void DrawCategory(Control control, FontItem fontItem, DrawItemEventArgs e)
    {
      Font font = new Font(defaultFontName, 10); 

      e.Graphics.FillRectangle(new SolidBrush(
        SystemColors.ButtonFace),
        e.Bounds.X, e.Bounds.Y + 5,
        e.Bounds.Width, e.Bounds.Height - 10);
      e.Graphics.FillRectangle(new SolidBrush(
        SystemColors.ButtonShadow),
        e.Bounds.X, e.Bounds.Y + e.Bounds.Height - 5,
        e.Bounds.Width, 1);

      StringFormat format1 = new StringFormat(StringFormatFlags.NoWrap);
      format1.LineAlignment = StringAlignment.Center;
      format1.Alignment = StringAlignment.Near;

      e.Graphics.DrawString(fontItem.Description, font, Brushes.Gray, e.Bounds, format1);
    } 
  }
  #endregion
}
